@@ -3,7 +3,7 @@
  *
  * @brief
  *
- * @copyright BSD License (see doc/LICENSE.md or http://www.libelektra.org)
+ * @copyright BSD License (see LICENSE.md or https://www.libelektra.org)
  */
 
 #include <kdb.h>
@@ -29,7 +29,7 @@ KeySet * doElektraMerge (KeySet * ours, KeySet * theirs, KeySet * base)
 }
 
 
-int main ()
+int main (void)
 {
 	// clang-format off
 //! [set]
