@@ -9,9 +9,13 @@ namespace ckdb
 extern "C" {
 #endif
 
-int elektraArrayIncName (Key * key);
 int elektraKsFilter (KeySet * result, KeySet * input, int (*filter) (const Key * k, void * argument), void * argument);
 
+int elektraArrayIncName (Key * key);
+int elektraArrayDecName (Key * key);
+
+int elektraArrayValidateName (const Key * key);
+
 const char * elektraKeyGetRelativeName (Key const * cur, Key const * parentKey);
 
 KeySet * elektraArrayGet (const Key * arrayParent, KeySet * keys);
