@@ -3,7 +3,7 @@
  *
  * @brief
  *
- * @copyright BSD License (see doc/LICENSE.md or http://www.libelektra.org)
+ * @copyright BSD License (see LICENSE.md or https://www.libelektra.org)
  */
 
 #ifndef KDBMERGE_HPP_
@@ -29,7 +29,7 @@ class MergingKDB : public KDB
 {
 public:
 	MergingKDB ();
-	MergingKDB (KDB & kdb);
+	explicit MergingKDB (KDB & kdb);
 	virtual ~MergingKDB () throw ();
 
 	/**
