@@ -11,6 +11,7 @@
 
 #include <kdberrors.h>
 #include <kdbhelper.h>
+#include <kdblogger.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -114,15 +115,18 @@ static void normalize (Key * key, Key * parentKey, BoolData * data)
 	if (isTrue (value, trueStrings))
 	{
 		keySetMeta (key, "origvalue", keyString (key));
+		ELEKTRA_LOG_DEBUG ("Convert “%s” to “%s”", value, data->true);
 		keySetString (key, data->true);
 	}
 	else if (isFalse (value, falseStrings))
 	{
 		keySetMeta (key, "origvalue", keyString (key));
+		ELEKTRA_LOG_DEBUG ("Convert “%s” to “%s”", value, data->false);
 		keySetString (key, data->false);
 	}
 	else
 	{
+		ELEKTRA_LOG_DEBUG ("Neither true nor false value");
 		keySetMeta (key, "boolean/invalid", "");
 		switch ((data->invalid & ~(WARNING)))
 		{
@@ -133,7 +137,7 @@ static void normalize (Key * key, Key * parentKey, BoolData * data)
 						      "Key %s with value %s is not a valid boolean. Defaulting to %s.", keyName (key),
 						      keyString (key), data->true);
 			}
-			keySetMeta (key, "originalvalue", keyString (key));
+			keySetMeta (key, "origvalue", keyString (key));
 			keySetString (key, data->true);
 			break;
 		case FALSE:
@@ -249,7 +253,11 @@ static void parseConfig (KeySet * config, BoolData * data)
 static int isBool (const Key * key)
 {
 	const Key * boolMeta = keyGetMeta (key, "type");
-	if (boolMeta && !strcmp (keyString (boolMeta), "boolean")) return 1;
+	if (boolMeta)
+	{
+		ELEKTRA_LOG_DEBUG ("Meta key “type” contains value “%s”", keyString (boolMeta));
+		return !strcmp (keyString (boolMeta), "boolean");
+	}
 
 	boolMeta = keyGetMeta (key, "check/type");
 	if (boolMeta && !strcmp (keyString (boolMeta), "boolean")) return 1;
@@ -286,7 +294,9 @@ int elektraBooleanGet (Plugin * handle ELEKTRA_UNUSED, KeySet * returned ELEKTRA
 	ksRewind (returned);
 	while ((key = ksNext (returned)) != NULL)
 	{
-		if (isBool (key))
+		uint8_t isBoolean = isBool (key);
+		ELEKTRA_LOG_DEBUG ("Key “%s” %s a boolean", keyName (key), isBoolean ? "contains" : "does not contain");
+		if (isBoolean)
 		{
 			normalize (key, parentKey, data);
 		}
@@ -294,11 +304,10 @@ int elektraBooleanGet (Plugin * handle ELEKTRA_UNUSED, KeySet * returned ELEKTRA
 	return 1; // success
 }
 
-static void restoreValue (Key * key, const char * origName)
+static void restoreValue (Key * key, const char * origValue)
 {
-	keySetString (key, origName);
+	keySetString (key, origValue);
 	keySetMeta (key, "origvalue", 0);
-	return;
 }
 
 int elektraBooleanSet (Plugin * handle ELEKTRA_UNUSED, KeySet * returned ELEKTRA_UNUSED, Key * parentKey ELEKTRA_UNUSED)
@@ -307,7 +316,13 @@ int elektraBooleanSet (Plugin * handle ELEKTRA_UNUSED, KeySet * returned ELEKTRA
 	// this function is optional
 
 	BoolData * data = elektraPluginGetData (handle);
-	if (!data) return -1;
+	if (!data)
+	{
+		KeySet * config = elektraPluginGetConfig (handle);
+		data = elektraCalloc (sizeof (BoolData));
+		parseConfig (config, data);
+		elektraPluginSetData (handle, data);
+	}
 	const char * trueValue = data->true;
 	const char * falseValue = data->false;
 
@@ -316,18 +331,21 @@ int elektraBooleanSet (Plugin * handle ELEKTRA_UNUSED, KeySet * returned ELEKTRA
 	int retVal = 1;
 	while ((key = ksNext (returned)) != NULL)
 	{
-		if (isBool (key))
+		uint8_t isBoolean = isBool (key);
+		ELEKTRA_LOG_DEBUG ("Key “%s” %s a boolean", keyName (key), isBoolean ? "contains" : "does not contain");
+		if (isBoolean)
 		{
-			const Key * nameMeta = keyGetMeta (key, "origvalue");
-			if ((!(!strcmp (keyString (key), trueValue) || !strcmp (keyString (key), falseValue))) ||
+			if (!keyGetMeta (key, "origvalue")) normalize (key, parentKey, data);
+			const Key * originalValue = keyGetMeta (key, "origvalue");
+			if (!(!strcmp (keyString (key), trueValue) || !strcmp (keyString (key), falseValue)) ||
 			    (keyGetMeta (key, "boolean/invalid")))
 			{
 				keySetMeta (key, "boolean/invalid", 0);
 				ELEKTRA_SET_ERRORF (ELEKTRA_ERROR_INVALID_BOOL, parentKey, "%s is not a valid boolean value",
-						    keyString (nameMeta));
+						    keyString (originalValue));
 				retVal = -1;
 			}
-			if (nameMeta) restoreValue (key, keyString (nameMeta));
+			if (originalValue) restoreValue (key, keyString (originalValue));
 		}
 	}
 	return retVal; // success
