@@ -3,7 +3,7 @@
  *
  * @brief Source for conditionals plugin
  *
- * @copyright BSD License (see doc/LICENSE.md or http://www.libelektra.org)
+ * @copyright BSD License (see LICENSE.md or https://www.libelektra.org)
  *
  */
 
@@ -16,6 +16,8 @@
 #include <errno.h>
 #include <kdbease.h>
 #include <kdberrors.h>
+#include <kdbmeta.h>
+#include <kdbproposal.h> //keyRel2
 #include <math.h>
 #include <regex.h>
 #include <stdio.h>
@@ -326,7 +328,7 @@ static CondResult evalCondition (const Key * curKey, const char * leftSide, Comp
 		if (ret == 0 && !strcmp (leftSide, "'1'")) result = TRUE;
 		break;
 	case OR:
-		if (!strcmp (leftSide, "'1'") || !strcmp (rightSide, "'1'")) result = TRUE;
+		if (!strcmp (leftSide, "'1'") || (rightSide && !strcmp (rightSide, "'1'"))) result = TRUE;
 		break;
 	default:
 		result = ERROR;
@@ -447,10 +449,9 @@ static CondResult parseSingleCondition (const Key * key, const char * condition,
 		--ptr;
 		++endPos;
 	}
-	char * leftSide = NULL;
-	char * rightSide = NULL;
 	int len = opStr - condition - endPos - startPos + 2;
-	leftSide = elektraMalloc (len);
+	char * leftSide = elektraMalloc (len);
+	char * rightSide = NULL;
 	strncpy (leftSide, condition + startPos, len - 2);
 	leftSide[len - 2] = '\0';
 	startPos = 0;
@@ -662,14 +663,11 @@ static CondResult parseConditionString (const Key * meta, const Key * suffixList
 		ksDel (ks);
 		return ERROR;
 	}
-	char * condition = NULL;
+	int startPos = m[1].rm_so;
+	int endPos = m[1].rm_eo;
+	char * condition = elektraMalloc (endPos - startPos + 1);
 	char * thenexpr = NULL;
 	char * elseexpr = NULL;
-	int startPos;
-	int endPos;
-	startPos = m[1].rm_so;
-	endPos = m[1].rm_eo;
-	condition = elektraMalloc (endPos - startPos + 1);
 	strncpy (condition, conditionString + startPos, endPos - startPos);
 	condition[endPos - startPos] = '\0';
 	nomatch = regexec (&regex2, conditionString, subMatches, m, 0);
@@ -830,10 +828,6 @@ static CondResult evaluateKey (const Key * meta, const Key * suffixList, Key * p
 	{
 		return TRUE;
 	}
-	else if (result != ERROR && op == ASSIGN)
-	{
-		return TRUE;
-	}
 	else if (result == NOEXPR)
 	{
 		return NOEXPR;
@@ -841,6 +835,52 @@ static CondResult evaluateKey (const Key * meta, const Key * suffixList, Key * p
 	return TRUE;
 }
 
+static CondResult evalMultipleConditions (Key * key, const Key * meta, const Key * suffixList, Key * parentKey, KeySet * returned)
+{
+	int countSucceeded = 0;
+	int countFailed = 0;
+	int countNoexpr = 0;
+	KeySet * condKS = elektraMetaArrayToKS (key, keyName (meta));
+	Key * c;
+	CondResult result = FALSE;
+	while ((c = ksNext (condKS)) != NULL)
+	{
+		if (!keyCmp (c, meta)) continue;
+		result = evaluateKey (c, suffixList, parentKey, key, returned, CONDITION);
+		if (result == TRUE)
+			++countSucceeded;
+		else if (result == ERROR)
+			++countFailed;
+		else if (result == NOEXPR)
+			++countNoexpr;
+	}
+	ksDel (condKS);
+	if (!strcmp (keyBaseName (meta), "all"))
+	{
+		// all conditions must evaluate to TRUE
+		if (countFailed || countNoexpr)
+			return ERROR;
+		else
+			return TRUE;
+	}
+	else if (!strcmp (keyBaseName (meta), "any"))
+	{
+		// at least one conditional must evaluate to TRUE
+		if (countSucceeded)
+			return TRUE;
+		else
+			return ERROR;
+	}
+	else
+	{
+		// no condition must evaluate to FALSE
+		if (countFailed)
+			return ERROR;
+		else
+			return TRUE;
+	}
+}
+
 int elektraConditionalsGet (Plugin * handle ELEKTRA_UNUSED, KeySet * returned ELEKTRA_UNUSED, Key * parentKey ELEKTRA_UNUSED)
 {
 	if (!strcmp (keyName (parentKey), "system/elektra/modules/conditionals"))
@@ -865,9 +905,14 @@ int elektraConditionalsGet (Plugin * handle ELEKTRA_UNUSED, KeySet * returned EL
 		Key * conditionMeta = (Key *)keyGetMeta (cur, "check/condition");
 		Key * assignMeta = (Key *)keyGetMeta (cur, "assign/condition");
 		Key * suffixList = (Key *)keyGetMeta (cur, "condition/validsuffix");
+		Key * anyConditionMeta = (Key *)keyGetMeta (cur, "check/condition/any");
+		Key * allConditionMeta = (Key *)keyGetMeta (cur, "check/condition/all");
+		Key * noneConditionMeta = (Key *)keyGetMeta (cur, "check/condition/none");
+
 		if (conditionMeta)
 		{
 			CondResult result;
+
 			result = evaluateKey (conditionMeta, suffixList, parentKey, cur, returned, CONDITION);
 			if (result == NOEXPR)
 			{
@@ -878,9 +923,55 @@ int elektraConditionalsGet (Plugin * handle ELEKTRA_UNUSED, KeySet * returned EL
 				ret |= result;
 			}
 		}
+		else if (allConditionMeta)
+		{
+			CondResult result;
+			result = evalMultipleConditions (cur, allConditionMeta, suffixList, parentKey, returned);
+			ret |= result;
+		}
+		else if (anyConditionMeta)
+		{
+			CondResult result;
+			result = evalMultipleConditions (cur, anyConditionMeta, suffixList, parentKey, returned);
+			ret |= result;
+		}
+		else if (noneConditionMeta)
+		{
+			CondResult result;
+			result = evalMultipleConditions (cur, noneConditionMeta, suffixList, parentKey, returned);
+			ret |= result;
+		}
+
 		if (assignMeta)
 		{
-			ret |= evaluateKey (assignMeta, suffixList, parentKey, cur, returned, ASSIGN);
+			if (keyString (assignMeta)[0] == '#')
+			{
+				KeySet * assignKS = elektraMetaArrayToKS (cur, "assign/condition");
+				Key * a;
+				while ((a = ksNext (assignKS)) != NULL)
+				{
+					if (keyCmp (a, assignMeta) == 0) continue;
+					CondResult result = evaluateKey (a, suffixList, parentKey, cur, returned, ASSIGN);
+					if (result == TRUE)
+					{
+						ret |= TRUE;
+						break;
+					}
+					else if (result == NOEXPR)
+					{
+						ret |= TRUE;
+					}
+					else
+					{
+						ret |= ERROR;
+					}
+				}
+				ksDel (assignKS);
+			}
+			else
+			{
+				ret |= evaluateKey (assignMeta, suffixList, parentKey, cur, returned, ASSIGN);
+			}
 		}
 	}
 	if (ret == TRUE) keySetMeta (parentKey, "error", 0);
@@ -898,9 +989,14 @@ int elektraConditionalsSet (Plugin * handle ELEKTRA_UNUSED, KeySet * returned EL
 		Key * conditionMeta = (Key *)keyGetMeta (cur, "check/condition");
 		Key * assignMeta = (Key *)keyGetMeta (cur, "assign/condition");
 		Key * suffixList = (Key *)keyGetMeta (cur, "condition/validsuffix");
+		Key * anyConditionMeta = (Key *)keyGetMeta (cur, "check/condition/any");
+		Key * allConditionMeta = (Key *)keyGetMeta (cur, "check/condition/all");
+		Key * noneConditionMeta = (Key *)keyGetMeta (cur, "check/condition/none");
+
 		if (conditionMeta)
 		{
 			CondResult result;
+
 			result = evaluateKey (conditionMeta, suffixList, parentKey, cur, returned, CONDITION);
 			if (result == NOEXPR)
 			{
@@ -911,9 +1007,55 @@ int elektraConditionalsSet (Plugin * handle ELEKTRA_UNUSED, KeySet * returned EL
 				ret |= result;
 			}
 		}
+		else if (allConditionMeta)
+		{
+			CondResult result;
+			result = evalMultipleConditions (cur, allConditionMeta, suffixList, parentKey, returned);
+			ret |= result;
+		}
+		else if (anyConditionMeta)
+		{
+			CondResult result;
+			result = evalMultipleConditions (cur, anyConditionMeta, suffixList, parentKey, returned);
+			ret |= result;
+		}
+		else if (noneConditionMeta)
+		{
+			CondResult result;
+			result = evalMultipleConditions (cur, noneConditionMeta, suffixList, parentKey, returned);
+			ret |= result;
+		}
+
 		if (assignMeta)
 		{
-			ret |= evaluateKey (assignMeta, suffixList, parentKey, cur, returned, ASSIGN);
+			if (keyString (assignMeta)[0] == '#')
+			{
+				KeySet * assignKS = elektraMetaArrayToKS (cur, "assign/condition");
+				Key * a;
+				while ((a = ksNext (assignKS)) != NULL)
+				{
+					if (keyCmp (a, assignMeta) == 0) continue;
+					CondResult result = evaluateKey (a, suffixList, parentKey, cur, returned, ASSIGN);
+					if (result == TRUE)
+					{
+						ret |= TRUE;
+						break;
+					}
+					else if (result == NOEXPR)
+					{
+						ret |= TRUE;
+					}
+					else
+					{
+						ret |= ERROR;
+					}
+				}
+				ksDel (assignKS);
+			}
+			else
+			{
+				ret |= evaluateKey (assignMeta, suffixList, parentKey, cur, returned, ASSIGN);
+			}
 		}
 	}
 	if (ret == TRUE) keySetMeta (parentKey, "error", 0);
