@@ -3,16 +3,18 @@
  *
  * @brief cryptographic interface using the Botan library
  *
- * @copyright BSD License (see doc/LICENSE.md or http://www.libelektra.org)
+ * @copyright BSD License (see LICENSE.md or https://www.libelektra.org)
  *
  */
 
 #include <botan/auto_rng.h>
 #include <botan/filters.h>
+#include <botan/hmac.h>
 #include <botan/init.h>
 #include <botan/lookup.h>
 #include <botan/pbkdf2.h>
 #include <botan/pipe.h>
+#include <botan/sha2_32.h>
 #include <botan/symkey.h>
 #include <kdbplugin.h>
 #include <memory>
@@ -28,6 +30,7 @@ extern "C" {
 #include "gpg.h"
 #include "helper.h"
 #include <base64_functions.h>
+#include <kdbassert.h>
 #include <kdberrors.h>
 #include <string.h>
 
@@ -35,19 +38,21 @@ extern "C" {
  * @brief derive the cryptographic key and IV for a given (Elektra) Key k
  * @param config KeySet holding the plugin/backend configuration
  * @param errorKey holds an error description in case of failure
+ * @param masterKey holds the decrypted master password from the plugin configuration
  * @param k the (Elektra)-Key to be encrypted
  * @param cKey holds a unique pointer to an allocated SymmetricKey.
  * @param cIv holds a unique pointer to an allocated InitializationVector.
  * @retval -1 on failure. errorKey holds the error description.
  * @retval 1 on success
  */
-static int getKeyIvForEncryption (KeySet * config, Key * errorKey, Key * k, unique_ptr<SymmetricKey> & cKey,
+static int getKeyIvForEncryption (KeySet * config, Key * errorKey, Key * masterKey, Key * k, unique_ptr<SymmetricKey> & cKey,
 				  unique_ptr<InitializationVector> & cIv)
 {
 	byte salt[ELEKTRA_CRYPTO_DEFAULT_SALT_LEN];
 	char * saltHexString = NULL;
 	const size_t requiredKeyBytes = ELEKTRA_CRYPTO_BOTAN_KEYSIZE + ELEKTRA_CRYPTO_BOTAN_BLOCKSIZE;
-	Key * msg = NULL;
+
+	ELEKTRA_ASSERT (masterKey != NULL, "Parameter `masterKey` must not be NULL");
 
 	try
 	{
@@ -66,26 +71,20 @@ static int getKeyIvForEncryption (KeySet * config, Key * errorKey, Key * k, uniq
 		// read iteration count
 		const kdb_unsigned_long_t iterations = CRYPTO_PLUGIN_FUNCTION (getIterationCount) (errorKey, config);
 
-		// receive master password from the configuration
-		msg = CRYPTO_PLUGIN_FUNCTION (getMasterPassword) (errorKey, config);
-		if (!msg) return -1; // error set by CRYPTO_PLUGIN_FUNCTION(getMasterPassword)()
-
 		// generate/derive the cryptographic key and the IV
-		unique_ptr<PBKDF> pbkdf = unique_ptr<PBKDF> (get_pbkdf ("PBKDF2(SHA-256)"));
-		OctetString derived = pbkdf->derive_key (
-			requiredKeyBytes, std::string (reinterpret_cast<const char *> (keyValue (msg)), keyGetValueSize (msg)), salt,
-			sizeof (salt), iterations);
+		PKCS5_PBKDF2 pbkdf (new HMAC (new SHA_256));
+		OctetString derived = pbkdf.derive_key (
+			requiredKeyBytes, std::string (reinterpret_cast<const char *> (keyValue (masterKey)), keyGetValueSize (masterKey)),
+			salt, sizeof (salt), iterations);
 
 		cKey = unique_ptr<SymmetricKey> (new SymmetricKey (derived.begin (), ELEKTRA_CRYPTO_BOTAN_KEYSIZE));
 		cIv = unique_ptr<InitializationVector> (
 			new InitializationVector (derived.begin () + ELEKTRA_CRYPTO_BOTAN_KEYSIZE, ELEKTRA_CRYPTO_BOTAN_BLOCKSIZE));
 
-		keyDel (msg);
 		return 1;
 	}
 	catch (std::exception & e)
 	{
-		if (msg) keyDel (msg);
 		ELEKTRA_SET_ERRORF (ELEKTRA_ERROR_CRYPTO_INIT, errorKey, "Failed to create a cryptographic key for encryption because: %s",
 				    e.what ());
 		return -1;
@@ -96,19 +95,22 @@ static int getKeyIvForEncryption (KeySet * config, Key * errorKey, Key * k, uniq
  * @brief derive the cryptographic key and IV for a given (Elektra) Key k
  * @param config KeySet holding the plugin/backend configuration
  * @param errorKey holds an error description in case of failure
+ * @param masterKey holds the decrypted master password from the plugin configuration
  * @param k the (Elektra)-Key to be encrypted
  * @param cKey holds a unique pointer to an allocated SymmetricKey.
  * @param cIv holds a unique pointer to an allocated InitializationVector.
  * @retval -1 on failure. errorKey holds the error description.
  * @retval 1 on success
  */
-static int getKeyIvForDecryption (KeySet * config, Key * errorKey, Key * k, unique_ptr<SymmetricKey> & cKey,
+static int getKeyIvForDecryption (KeySet * config, Key * errorKey, Key * masterKey, Key * k, unique_ptr<SymmetricKey> & cKey,
 				  unique_ptr<InitializationVector> & cIv)
 {
 	const size_t requiredKeyBytes = ELEKTRA_CRYPTO_BOTAN_KEYSIZE + ELEKTRA_CRYPTO_BOTAN_BLOCKSIZE;
 	kdb_octet_t * saltBuffer;
 	kdb_unsigned_long_t saltBufferLen = 0;
 
+	ELEKTRA_ASSERT (masterKey != NULL, "Parameter `masterKey` must not be NULL");
+
 	// get the salt
 	if (CRYPTO_PLUGIN_FUNCTION (getSaltFromPayload) (errorKey, k, &saltBuffer, &saltBufferLen) != 1)
 	{
@@ -118,32 +120,22 @@ static int getKeyIvForDecryption (KeySet * config, Key * errorKey, Key * k, uniq
 	// get the iteration count
 	const kdb_unsigned_long_t iterations = CRYPTO_PLUGIN_FUNCTION (getIterationCount) (errorKey, config);
 
-	// receive master password from the configuration
-	Key * msg = CRYPTO_PLUGIN_FUNCTION (getMasterPassword) (errorKey, config);
-	if (!msg)
-	{
-		elektraFree (saltBuffer);
-		return -1; // error set by CRYPTO_PLUGIN_FUNCTION(getMasterPassword)()
-	}
-
 	try
 	{
 		// derive the cryptographic key and the IV
-		unique_ptr<PBKDF> pbkdf = unique_ptr<PBKDF> (get_pbkdf ("PBKDF2(SHA-256)"));
-		OctetString derived = pbkdf->derive_key (
-			requiredKeyBytes, std::string (reinterpret_cast<const char *> (keyValue (msg)), keyGetValueSize (msg)), saltBuffer,
-			saltBufferLen, iterations);
+		PKCS5_PBKDF2 pbkdf (new HMAC (new SHA_256));
+		OctetString derived = pbkdf.derive_key (
+			requiredKeyBytes, std::string (reinterpret_cast<const char *> (keyValue (masterKey)), keyGetValueSize (masterKey)),
+			saltBuffer, saltBufferLen, iterations);
 
 		cKey = unique_ptr<SymmetricKey> (new SymmetricKey (derived.begin (), ELEKTRA_CRYPTO_BOTAN_KEYSIZE));
 		cIv = unique_ptr<InitializationVector> (
 			new InitializationVector (derived.begin () + ELEKTRA_CRYPTO_BOTAN_KEYSIZE, ELEKTRA_CRYPTO_BOTAN_BLOCKSIZE));
 
-		keyDel (msg);
 		return 1;
 	}
 	catch (std::exception & e)
 	{
-		if (msg) keyDel (msg);
 		ELEKTRA_SET_ERRORF (ELEKTRA_ERROR_CRYPTO_INIT, errorKey,
 				    "Failed to restore the cryptographic key for decryption because: %s", e.what ());
 		return -1;
@@ -164,12 +156,12 @@ int elektraCryptoBotanInit (Key * errorKey)
 	return 1; // success
 }
 
-int elektraCryptoBotanEncrypt (KeySet * pluginConfig, Key * k, Key * errorKey)
+int elektraCryptoBotanEncrypt (KeySet * pluginConfig, Key * k, Key * errorKey, Key * masterKey)
 {
 	// get cryptographic material
 	unique_ptr<SymmetricKey> cryptoKey;
 	unique_ptr<InitializationVector> cryptoIv;
-	if (getKeyIvForEncryption (pluginConfig, errorKey, k, cryptoKey, cryptoIv) != 1)
+	if (getKeyIvForEncryption (pluginConfig, errorKey, masterKey, k, cryptoKey, cryptoIv) != 1)
 	{
 		return -1;
 	}
@@ -224,13 +216,20 @@ int elektraCryptoBotanEncrypt (KeySet * pluginConfig, Key * k, Key * errorKey)
 		const size_t msgLength = encryptor.remaining ();
 		if (msgLength > 0)
 		{
-			auto buffer = unique_ptr<byte[]>{ new byte[msgLength + sizeof (kdb_unsigned_long_t) + saltLen] };
+			auto buffer = unique_ptr<byte[]>{
+				new byte[msgLength + ELEKTRA_CRYPTO_MAGIC_NUMBER_LEN + sizeof (kdb_unsigned_long_t) + saltLen]
+			};
+			size_t bufferIndex = 0;
 
-			memcpy (&buffer[0], &saltLen, sizeof (kdb_unsigned_long_t));
-			memcpy (&buffer[sizeof (kdb_unsigned_long_t)], salt, saltLen);
+			memcpy (&buffer[bufferIndex], ELEKTRA_CRYPTO_MAGIC_NUMBER, ELEKTRA_CRYPTO_MAGIC_NUMBER_LEN);
+			bufferIndex += ELEKTRA_CRYPTO_MAGIC_NUMBER_LEN;
+			memcpy (&buffer[bufferIndex], &saltLen, sizeof (kdb_unsigned_long_t));
+			bufferIndex += sizeof (kdb_unsigned_long_t);
+			memcpy (&buffer[bufferIndex], salt, saltLen);
+			bufferIndex += saltLen;
 
-			const size_t buffered = encryptor.read (&buffer[sizeof (kdb_unsigned_long_t) + saltLen], msgLength);
-			keySetBinary (k, &buffer[0], buffered + sizeof (kdb_unsigned_long_t) + saltLen);
+			const size_t buffered = encryptor.read (&buffer[bufferIndex], msgLength);
+			keySetBinary (k, &buffer[0], buffered + bufferIndex);
 		}
 	}
 	catch (std::exception & e)
@@ -244,12 +243,12 @@ int elektraCryptoBotanEncrypt (KeySet * pluginConfig, Key * k, Key * errorKey)
 	return 1; // success
 }
 
-int elektraCryptoBotanDecrypt (KeySet * pluginConfig, Key * k, Key * errorKey)
+int elektraCryptoBotanDecrypt (KeySet * pluginConfig, Key * k, Key * errorKey, Key * masterKey)
 {
 	// get cryptographic material
 	unique_ptr<SymmetricKey> cryptoKey;
 	unique_ptr<InitializationVector> cryptoIv;
-	if (getKeyIvForDecryption (pluginConfig, errorKey, k, cryptoKey, cryptoIv) != 1)
+	if (getKeyIvForDecryption (pluginConfig, errorKey, masterKey, k, cryptoKey, cryptoIv) != 1)
 	{
 		return -1;
 	}
@@ -263,8 +262,8 @@ int elektraCryptoBotanDecrypt (KeySet * pluginConfig, Key * k, Key * errorKey)
 	saltLen += sizeof (kdb_unsigned_long_t);
 
 	// set payload pointer
-	const byte * payload = reinterpret_cast<const byte *> (keyValue (k)) + saltLen;
-	const size_t payloadLen = keyGetValueSize (k) - saltLen;
+	const byte * payload = reinterpret_cast<const byte *> (keyValue (k)) + saltLen + ELEKTRA_CRYPTO_MAGIC_NUMBER_LEN;
+	const size_t payloadLen = keyGetValueSize (k) - saltLen - ELEKTRA_CRYPTO_MAGIC_NUMBER_LEN;
 
 	try
 	{
@@ -272,13 +271,17 @@ int elektraCryptoBotanDecrypt (KeySet * pluginConfig, Key * k, Key * errorKey)
 		Pipe decryptor (get_cipher (ELEKTRA_CRYPTO_BOTAN_ALGORITHM, *cryptoKey, *cryptoIv, DECRYPTION));
 		kdb_octet_t flags = ELEKTRA_CRYPTO_FLAG_NONE;
 
-		// decrypt the conent
+		// decrypt the content
 		decryptor.process_msg (payload, payloadLen);
 
 		if (decryptor.remaining () > 0)
 		{
 			// decode the "header" flags
-			decryptor.read (static_cast<byte *> (&flags), sizeof (kdb_octet_t));
+			const size_t flagLength = decryptor.read (static_cast<byte *> (&flags), sizeof (kdb_octet_t));
+			if (flagLength != sizeof (kdb_octet_t))
+			{
+				throw std::length_error ("Failed to restore the original data type of the value.");
+			}
 		}
 
 		const size_t msgLength = decryptor.remaining ();
