@@ -3,7 +3,7 @@
  *
  * @brief cryptographic interface using the gcrypt library
  *
- * @copyright BSD License (see doc/LICENSE.md or http://www.libelektra.org)
+ * @copyright BSD License (see LICENSE.md or https://www.libelektra.org)
  *
  */
 
@@ -16,6 +16,7 @@
 #include <base64_functions.h>
 #include <errno.h>
 #include <gcrypt.h>
+#include <kdbassert.h>
 #include <kdberrors.h>
 #include <kdbtypes.h>
 #include <pthread.h>
@@ -32,19 +33,22 @@ GCRY_THREAD_OPTION_PTHREAD_IMPL;
  * @brief derive the cryptographic key and IV for a given (Elektra) Key k
  * @param config KeySet holding the plugin/backend configuration
  * @param errorKey holds an error description in case of failure
+ * @param masterKey holds the decrypted master password from the plugin configuration
  * @param k the (Elektra)-Key to be encrypted
  * @param cKey (Elektra)-Key holding the cryptographic material
  * @param cIv (Elektra)-Key holding the initialization vector
  * @retval -1 on failure. errorKey holds the error description.
  * @retval 1 on success
  */
-static int getKeyIvForEncryption (KeySet * config, Key * errorKey, Key * k, Key * cKey, Key * cIv)
+static int getKeyIvForEncryption (KeySet * config, Key * errorKey, Key * masterKey, Key * k, Key * cKey, Key * cIv)
 {
 	gcry_error_t gcry_err;
 	kdb_octet_t salt[ELEKTRA_CRYPTO_DEFAULT_SALT_LEN];
 	kdb_octet_t keyBuffer[KEY_BUFFER_SIZE];
 	char * saltHexString = NULL;
 
+	ELEKTRA_ASSERT (masterKey != NULL, "Parameter `masterKey` must not be NULL");
+
 	// generate the salt
 	gcry_create_nonce (salt, sizeof (salt));
 	saltHexString = ELEKTRA_PLUGIN_FUNCTION (ELEKTRA_PLUGIN_NAME_C, base64Encode) (salt, sizeof (salt));
@@ -59,47 +63,40 @@ static int getKeyIvForEncryption (KeySet * config, Key * errorKey, Key * k, Key
 	// read iteration count
 	const kdb_unsigned_long_t iterations = CRYPTO_PLUGIN_FUNCTION (getIterationCount) (errorKey, config);
 
-	// receive master password from the configuration
-	Key * msg = CRYPTO_PLUGIN_FUNCTION (getMasterPassword) (errorKey, config);
-	if (!msg) goto error; // error set by CRYPTO_PLUGIN_FUNCTION(getMasterPassword)()
-
 	// generate/derive the cryptographic key and the IV
-	if ((gcry_err = gcry_kdf_derive (keyValue (msg), keyGetValueSize (msg), GCRY_KDF_PBKDF2, GCRY_MD_SHA512, salt, sizeof (salt),
-					 iterations, KEY_BUFFER_SIZE, keyBuffer)))
+	if ((gcry_err = gcry_kdf_derive (keyValue (masterKey), keyGetValueSize (masterKey), GCRY_KDF_PBKDF2, GCRY_MD_SHA512, salt,
+					 sizeof (salt), iterations, KEY_BUFFER_SIZE, keyBuffer)))
 	{
 		ELEKTRA_SET_ERRORF (ELEKTRA_ERROR_CRYPTO_INTERNAL_ERROR, errorKey,
 				    "Failed to create a cryptographic key for encryption because: %s", gcry_strerror (gcry_err));
-		goto error;
+		return -1;
 	}
 
 	keySetBinary (cKey, keyBuffer, ELEKTRA_CRYPTO_GCRY_KEYSIZE);
 	keySetBinary (cIv, keyBuffer + ELEKTRA_CRYPTO_GCRY_KEYSIZE, ELEKTRA_CRYPTO_GCRY_BLOCKSIZE);
-
-	keyDel (msg);
 	return 1;
-
-error:
-	if (msg) keyDel (msg);
-	return -1;
 }
 
 /**
  * @brief derive the cryptographic key and IV for a given (Elektra) Key k
  * @param config KeySet holding the plugin/backend configuration
  * @param errorKey holds an error description in case of failure
+ * @param masterKey holds the decrypted master password from the plugin configuration
  * @param k the (Elektra)-Key to be encrypted
  * @param cKey (Elektra)-Key holding the cryptographic material
  * @param cIv (Elektra)-Key holding the initialization vector
  * @retval -1 on failure. errorKey holds the error description.
  * @retval 1 on success
  */
-static int getKeyIvForDecryption (KeySet * config, Key * errorKey, Key * k, Key * cKey, Key * cIv)
+static int getKeyIvForDecryption (KeySet * config, Key * errorKey, Key * masterKey, Key * k, Key * cKey, Key * cIv)
 {
 	gcry_error_t gcry_err;
 	kdb_octet_t keyBuffer[KEY_BUFFER_SIZE];
 	kdb_octet_t * saltBuffer = NULL;
 	kdb_unsigned_long_t saltBufferLen = 0;
 
+	ELEKTRA_ASSERT (masterKey != NULL, "Parameter `masterKey` must not be NULL");
+
 	// get the salt
 	if (CRYPTO_PLUGIN_FUNCTION (getSaltFromPayload) (errorKey, k, &saltBuffer, &saltBufferLen) != 1)
 	{
@@ -109,28 +106,18 @@ static int getKeyIvForDecryption (KeySet * config, Key * errorKey, Key * k, Key
 	// get the iteration count
 	const kdb_unsigned_long_t iterations = CRYPTO_PLUGIN_FUNCTION (getIterationCount) (errorKey, config);
 
-	// receive master password from the configuration
-	Key * msg = CRYPTO_PLUGIN_FUNCTION (getMasterPassword) (errorKey, config);
-	if (!msg) goto error; // error set by CRYPTO_PLUGIN_FUNCTION(getMasterPassword)()
-
 	// derive the cryptographic key and the IV
-	if ((gcry_err = gcry_kdf_derive (keyValue (msg), keyGetValueSize (msg), GCRY_KDF_PBKDF2, GCRY_MD_SHA512, saltBuffer, saltBufferLen,
-					 iterations, KEY_BUFFER_SIZE, keyBuffer)))
+	if ((gcry_err = gcry_kdf_derive (keyValue (masterKey), keyGetValueSize (masterKey), GCRY_KDF_PBKDF2, GCRY_MD_SHA512, saltBuffer,
+					 saltBufferLen, iterations, KEY_BUFFER_SIZE, keyBuffer)))
 	{
 		ELEKTRA_SET_ERRORF (ELEKTRA_ERROR_CRYPTO_INTERNAL_ERROR, errorKey,
 				    "Failed to restore the cryptographic key for decryption because: %s", gcry_strerror (gcry_err));
-		goto error;
+		return -1;
 	}
 
 	keySetBinary (cKey, keyBuffer, ELEKTRA_CRYPTO_GCRY_KEYSIZE);
 	keySetBinary (cIv, keyBuffer + ELEKTRA_CRYPTO_GCRY_KEYSIZE, ELEKTRA_CRYPTO_GCRY_BLOCKSIZE);
-
-	keyDel (msg);
 	return 1;
-
-error:
-	if (msg) keyDel (msg);
-	return -1;
 }
 
 void elektraCryptoGcryHandleDestroy (elektraCryptoHandle * handle)
@@ -166,7 +153,7 @@ int elektraCryptoGcryInit (Key * errorKey)
 	return 1;
 }
 
-int elektraCryptoGcryHandleCreate (elektraCryptoHandle ** handle, KeySet * config, Key * errorKey, Key * k,
+int elektraCryptoGcryHandleCreate (elektraCryptoHandle ** handle, KeySet * config, Key * errorKey, Key * masterKey, Key * k,
 				   const enum ElektraCryptoOperation op)
 {
 	gcry_error_t gcry_err;
@@ -181,7 +168,7 @@ int elektraCryptoGcryHandleCreate (elektraCryptoHandle ** handle, KeySet * confi
 	switch (op)
 	{
 	case ELEKTRA_CRYPTO_ENCRYPT:
-		if (getKeyIvForEncryption (config, errorKey, k, key, iv) != 1)
+		if (getKeyIvForEncryption (config, errorKey, masterKey, k, key, iv) != 1)
 		{
 			keyDel (key);
 			keyDel (iv);
@@ -190,7 +177,7 @@ int elektraCryptoGcryHandleCreate (elektraCryptoHandle ** handle, KeySet * confi
 		break;
 
 	case ELEKTRA_CRYPTO_DECRYPT:
-		if (getKeyIvForDecryption (config, errorKey, k, key, iv) != 1)
+		if (getKeyIvForDecryption (config, errorKey, masterKey, k, key, iv) != 1)
 		{
 			keyDel (key);
 			keyDel (iv);
@@ -298,6 +285,7 @@ int elektraCryptoGcryEncrypt (elektraCryptoHandle * handle, Key * k, Key * error
 		outputLen = (contentLen / ELEKTRA_CRYPTO_GCRY_BLOCKSIZE) + 2;
 	}
 	outputLen *= ELEKTRA_CRYPTO_GCRY_BLOCKSIZE;
+	outputLen += ELEKTRA_CRYPTO_MAGIC_NUMBER_LEN;
 	outputLen += sizeof (kdb_unsigned_long_t) + saltLen;
 	kdb_octet_t * output = elektraMalloc (outputLen);
 	if (!output)
@@ -307,9 +295,13 @@ int elektraCryptoGcryEncrypt (elektraCryptoHandle * handle, Key * k, Key * error
 		return -1;
 	}
 
-	// encode the salt into the crypto payload
 	kdb_octet_t * current = output;
 
+	// output of the magic number
+	memcpy (current, ELEKTRA_CRYPTO_MAGIC_NUMBER, ELEKTRA_CRYPTO_MAGIC_NUMBER_LEN);
+	current += ELEKTRA_CRYPTO_MAGIC_NUMBER_LEN;
+
+	// encode the salt into the crypto payload
 	memcpy (current, &saltLen, sizeof (kdb_unsigned_long_t));
 	current += sizeof (kdb_unsigned_long_t);
 	memcpy (current, salt, saltLen);
@@ -330,8 +322,9 @@ int elektraCryptoGcryEncrypt (elektraCryptoHandle * handle, Key * k, Key * error
 	current += ELEKTRA_CRYPTO_GCRY_BLOCKSIZE;
 
 	// encrypt the value using gcrypt's in-place encryption
-	const size_t dataLen = outputLen - ELEKTRA_CRYPTO_GCRY_BLOCKSIZE - sizeof (kdb_unsigned_long_t) - saltLen;
-	memcpy (current, content, contentLen);
+	const size_t dataLen =
+		outputLen - ELEKTRA_CRYPTO_GCRY_BLOCKSIZE - sizeof (kdb_unsigned_long_t) - saltLen - ELEKTRA_CRYPTO_MAGIC_NUMBER_LEN;
+	if (contentLen) memcpy (current, content, contentLen);
 	gcry_err = gcry_cipher_encrypt (*handle, current, dataLen, NULL, 0);
 	if (gcry_err != 0)
 	{
@@ -363,8 +356,8 @@ int elektraCryptoGcryDecrypt (elektraCryptoHandle * handle, Key * k, Key * error
 	saltLen += sizeof (kdb_unsigned_long_t);
 
 	// set payload pointer
-	const kdb_octet_t * payload = ((kdb_octet_t *)keyValue (k)) + saltLen;
-	const size_t payloadLen = keyGetValueSize (k) - saltLen;
+	const kdb_octet_t * payload = ((kdb_octet_t *)keyValue (k)) + saltLen + ELEKTRA_CRYPTO_MAGIC_NUMBER_LEN;
+	const size_t payloadLen = keyGetValueSize (k) - saltLen - ELEKTRA_CRYPTO_MAGIC_NUMBER_LEN;
 
 	// plausibility check
 	if (payloadLen % ELEKTRA_CRYPTO_GCRY_BLOCKSIZE != 0)
