@@ -4,7 +4,7 @@
  * @brief test suite for the crypto plugin.
  * Contains shared functions for all compile variants.
  *
- * @copyright BSD License (see doc/LICENSE.md or http://www.libelektra.org)
+ * @copyright BSD License (see LICENSE.md or https://www.libelektra.org)
  *
  */
 
@@ -55,7 +55,7 @@ static int isMarkedForEncryption (const Key * k)
 /**
  * @brief create a new KeySet holding sample data for encryption and decryption.
  */
-static KeySet * newTestdataKeySet ()
+static KeySet * newTestdataKeySet (void)
 {
 	Key * kUnchanged1 = keyNew ("user/crypto/test/nochange", KEY_END);
 	Key * kUnchanged2 = keyNew ("user/crypto/test/nochange2", KEY_END);
@@ -98,9 +98,9 @@ static inline void setPluginShutdown (KeySet * config)
 	ksAppendKey (config, keyNew (ELEKTRA_CRYPTO_PARAM_SHUTDOWN, KEY_VALUE, "1", 0));
 }
 
-static KeySet * newPluginConfiguration ()
+static KeySet * newPluginConfiguration (void)
 {
-	return ksNew (2, keyNew (ELEKTRA_CRYPTO_PARAM_GPG_KEY, KEY_VALUE, TEST_KEY_ID, KEY_END),
+	return ksNew (2, keyNew (ELEKTRA_RECIPIENT_KEY, KEY_VALUE, TEST_KEY_ID, KEY_END),
 		      keyNew (ELEKTRA_CRYPTO_PARAM_GPG_UNIT_TEST, KEY_VALUE, "1", KEY_END), KS_END);
 }
 
@@ -125,7 +125,6 @@ static void test_init (const char * pluginName)
 		succeed_if (plugin->kdbClose != 0, "no close pointer");
 		succeed_if (plugin->kdbGet != 0, "no get pointer");
 		succeed_if (plugin->kdbSet != 0, "no set pointer");
-		succeed_if (plugin->kdbError != 0, "no error pointer");
 
 		// try re-opening the plugin
 		succeed_if (plugin->kdbClose (plugin, parentKey) == 1, "kdb close failed");
