@@ -3,7 +3,7 @@
  *
  * @brief Source for dbus plugin
  *
- * @copyright BSD License (see doc/LICENSE.md or http://www.libelektra.org)
+ * @copyright BSD License (see LICENSE.md or https://www.libelektra.org)
  *
  */
 
@@ -79,19 +79,28 @@ int elektraDbusSet (Plugin * handle, KeySet * returned, Key * parentKey)
 		}
 	}
 
-	if (!strncmp (keyName (parentKey), "user", 4))
+	if (!strncmp (keyString (ksLookupByName (elektraPluginGetConfig (handle), "/announce", 0)), "once", 4))
 	{
-		announceKeys (addedKeys, "KeyAdded", DBUS_BUS_SESSION);
-		announceKeys (changedKeys, "KeyChanged", DBUS_BUS_SESSION);
-		announceKeys (removedKeys, "KeyDeleted", DBUS_BUS_SESSION);
+		if (!strncmp (keyName (parentKey), "user", 4)) elektraDbusSendMessage (DBUS_BUS_SESSION, keyName (parentKey), "Commit");
+		if (!strncmp (keyName (parentKey), "system", 6)) elektraDbusSendMessage (DBUS_BUS_SYSTEM, keyName (parentKey), "Commit");
 	}
-	else if (!strncmp (keyName (parentKey), "system", 6))
+	else
 	{
-		announceKeys (addedKeys, "KeyAdded", DBUS_BUS_SYSTEM);
-		announceKeys (changedKeys, "KeyChanged", DBUS_BUS_SYSTEM);
-		announceKeys (removedKeys, "KeyDeleted", DBUS_BUS_SYSTEM);
+		if (!strncmp (keyName (parentKey), "user", 4))
+		{
+			announceKeys (addedKeys, "KeyAdded", DBUS_BUS_SESSION);
+			announceKeys (changedKeys, "KeyChanged", DBUS_BUS_SESSION);
+			announceKeys (removedKeys, "KeyDeleted", DBUS_BUS_SESSION);
+		}
+		else if (!strncmp (keyName (parentKey), "system", 6))
+		{
+			announceKeys (addedKeys, "KeyAdded", DBUS_BUS_SYSTEM);
+			announceKeys (changedKeys, "KeyChanged", DBUS_BUS_SYSTEM);
+			announceKeys (removedKeys, "KeyDeleted", DBUS_BUS_SYSTEM);
+		}
 	}
 
+
 	ksDel (oldKeys);
 	ksDel (addedKeys);
 	ksDel (changedKeys);
