@@ -3,7 +3,7 @@
  *
  * @brief filter plugin providing cryptographic operations
  *
- * @copyright BSD License (see doc/LICENSE.md or http://www.libelektra.org)
+ * @copyright BSD License (see LICENSE.md or https://www.libelektra.org)
  *
  */
 
@@ -19,4 +19,9 @@ int ELEKTRA_PLUGIN_FUNCTION (ELEKTRA_PLUGIN_NAME, checkconf) (Key * errorKey, Ke
 
 Plugin * ELEKTRA_PLUGIN_EXPORT (crypto);
 
+#define ELEKTRA_FCRYPT_DEFAULT_TMPDIR "/tmp"
+
+#define ELEKTRA_FCRYPT_CONFIG_TEXTMODE "/fcrypt/textmode"
+#define ELEKTRA_FCRYPT_CONFIG_TMPDIR "/fcrypt/tmpdir"
+
 #endif
