@@ -156,6 +156,7 @@ int elektraJniOpen (Plugin * handle, Key * errorKey)
 	Data * data = elektraMalloc (sizeof (Data));
 	data->module = 0;
 	data->printException = 0;
+	elektraPluginSetData (handle, data);
 
 	KeySet * config = elektraPluginGetConfig (handle);
 	Key * k = ksLookupByName (config, "/module", 0);
@@ -294,12 +295,7 @@ int elektraJniOpen (Plugin * handle, Key * errorKey)
 		return -1;
 	}
 
-	int ret = call2Arg (data, config, errorKey, "open");
-	if (ret != -1)
-	{
-		elektraPluginSetData (handle, data);
-	}
-	return ret;
+	return call2Arg (data, config, errorKey, "open");
 }
 
 int elektraJniClose (Plugin * handle, Key * errorKey)
@@ -311,11 +307,6 @@ int elektraJniClose (Plugin * handle, Key * errorKey)
 		return 0;
 	}
 
-	if (!data)
-	{
-		return -1;
-	}
-
 	int ret = call1Arg (data, errorKey, "close");
 
 	(*data->jvm)->DestroyJavaVM (data->jvm);
@@ -349,11 +340,6 @@ int elektraJniGet (Plugin * handle, KeySet * returned, Key * parentKey)
 		return 0;
 	}
 
-	if (!data)
-	{
-		return -1;
-	}
-
 	return call2Arg (data, returned, parentKey, "get");
 }
 
