@@ -3,7 +3,7 @@
  *
  * @brief Source for list plugin
  *
- * @copyright BSD License (see doc/LICENSE.md or http://www.libelektra.org)
+ * @copyright BSD License (see LICENSE.md or https://www.libelektra.org)
  *
  */
 
@@ -286,7 +286,8 @@ static int runPlugins (KeySet * pluginKS, KeySet * modules, KeySet * plugins, Ke
 			ksDel (realPluginConfig);
 			if (!slave)
 			{
-				goto error;
+				ksDel (configOrig);
+				return -1;
 			}
 			Key * slaveKey = keyNew (name, KEY_BINARY, KEY_SIZE, sizeof (Plugin *), KEY_VALUE, &slave, KEY_END);
 			keySetName (slaveKey, "/");
@@ -295,43 +296,16 @@ static int runPlugins (KeySet * pluginKS, KeySet * modules, KeySet * plugins, Ke
 			keyDel (slaveKey);
 		}
 
-		if (op == GET)
+		if ((op == GET && slave->kdbGet && (slave->kdbGet (slave, returned, parentKey)) == -1) ||
+		    (op == SET && slave->kdbSet && (slave->kdbSet (slave, returned, parentKey)) == -1) ||
+		    (op == ERR && slave->kdbError && (slave->kdbError (slave, returned, parentKey)) == -1))
 		{
-			if (slave->kdbGet)
-			{
-				if ((slave->kdbGet (slave, returned, parentKey)) == -1)
-				{
-					goto error;
-				}
-			}
-		}
-		else if (op == SET)
-		{
-			if (slave->kdbSet)
-			{
-				if ((slave->kdbSet (slave, returned, parentKey)) == -1)
-				{
-					goto error;
-				}
-			}
-		}
-		else if (op == ERR)
-		{
-			if (slave->kdbError)
-			{
-				if ((slave->kdbError (slave, returned, parentKey)) == -1)
-				{
-					goto error;
-				}
-			}
+			ksDel (configOrig);
+			return -1;
 		}
 	}
 	ksDel (configOrig);
 	return 1;
-
-error:
-	ksDel (configOrig);
-	return -1;
 }
 
 int elektraListGet (Plugin * handle, KeySet * returned, Key * parentKey)
@@ -362,7 +336,7 @@ int elektraListGet (Plugin * handle, KeySet * returned, Key * parentKey)
 	ksRewind (pluginKS);
 	int ret = runPlugins (pluginKS, placements->modules, placements->plugins, ksDup (config), returned, parentKey, GET, ksNext);
 	placements->getCurrent = ((++currentPlacement) % getEnd);
-	while (!placements->getPlacements[currentPlacement])
+	while (currentPlacement < getEnd && !placements->getPlacements[currentPlacement])
 	{
 		placements->getCurrent = ((++currentPlacement) % getEnd);
 	}
@@ -380,7 +354,7 @@ int elektraListSet (Plugin * handle, KeySet * returned, Key * parentKey)
 	int ret = 0;
 	ret = runPlugins (pluginKS, placements->modules, placements->plugins, ksDup (config), returned, parentKey, SET, ksPop);
 	placements->setCurrent = ((++currentPlacement) % setEnd);
-	while (!placements->setPlacements[currentPlacement])
+	while (currentPlacement < setEnd && !placements->setPlacements[currentPlacement])
 	{
 		placements->setCurrent = ((++currentPlacement) % setEnd);
 	}
@@ -399,7 +373,7 @@ int elektraListError (Plugin * handle, KeySet * returned, Key * parentKey)
 	ksRewind (pluginKS);
 	int ret = runPlugins (pluginKS, placements->modules, placements->plugins, ksDup (config), returned, parentKey, ERR, ksPop);
 	placements->errCurrent = ((++currentPlacement) % errEnd);
-	while (!placements->errPlacements[currentPlacement])
+	while (currentPlacement < errEnd && !placements->errPlacements[currentPlacement])
 	{
 		placements->errCurrent = ((++currentPlacement) % errEnd);
 	}
@@ -414,8 +388,8 @@ int elektraListAddPlugin (Plugin * handle, KeySet * pluginConfig)
 		return 0;
 	}
 	ksRewind (pluginConfig);
+	ksNext (pluginConfig);
 	Key * lookup = ksNext (pluginConfig);
-	lookup = ksNext (pluginConfig);
 	if (keyBaseName (lookup)[0] != '#')
 	{
 		return -1;
@@ -442,8 +416,8 @@ int elektraListEditPlugin (Plugin * handle, KeySet * pluginConfig)
 		return 0;
 	}
 	ksRewind (pluginConfig);
+	ksNext (pluginConfig);
 	Key * lookup = ksNext (pluginConfig);
-	lookup = ksNext (pluginConfig);
 	if (keyBaseName (lookup)[0] != '#')
 	{
 		return -1;
