@@ -106,7 +106,7 @@ static inline Codes rvToRc (int rc)
 static int elektraResolveFilename (Key * parentKey, ElektraResolveTempfile tmpFile)
 {
 	int rc = 0;
-	ElektraInvokeHandle * handle = elektraInvokeInitialize ("resolver");
+	ElektraInvokeHandle * handle = elektraInvokeOpen ("resolver", 0);
 	if (!handle)
 	{
 		rc = -1;
