@@ -3,7 +3,7 @@
  *
  * @brief
  *
- * @copyright BSD License (see doc/LICENSE.md or http://www.libelektra.org)
+ * @copyright BSD License (see LICENSE.md or https://www.libelektra.org)
  */
 
 #include "xmltool.h"
@@ -150,7 +150,6 @@ ssize_t keyToStream (const Key * key, FILE * stream, option_t options)
 ssize_t keyToStreamBasename (const Key * key, FILE * stream, const char * parent, const size_t parentSize, option_t options)
 {
 	ssize_t written = 0;
-	char buffer[KDB_MAX_PATH_LENGTH];
 
 	/* Write key name */
 	if (parent)
@@ -174,6 +173,7 @@ ssize_t keyToStreamBasename (const Key * key, FILE * stream, const char * parent
 	{ /* no "<key basename=..." was written so far */
 		if (options & KDB_O_FULLNAME)
 		{
+			char buffer[KDB_MAX_PATH_LENGTH];
 			keyGetFullName (key, buffer, sizeof (buffer));
 			written += fprintf (stream, "<key name=\"%s\"", buffer);
 		}
@@ -336,9 +336,9 @@ ssize_t keyToStreamBasename (const Key * key, FILE * stream, const char * parent
  * @verbatim
 <?xml version="1.0" encoding="UTF-8"?>
 <!-- Generated by Elektra API. Total of n keys. -->
-<keyset xmlns="http://www.libelektra.org"
+<keyset xmlns="https://www.libelektra.org"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
-        xsi:schemaLocation="http://www.libelektra.org elektra.xsd">
+        xsi:schemaLocation="https://www.libelektra.org elektra.xsd">
  * @endverbatim
  *
  * @param ks the KeySet to serialise
@@ -374,26 +374,25 @@ ssize_t ksToStream (const KeySet * ks, FILE * stream, option_t options)
 {
 	size_t written = 0;
 	Key * key = 0;
-	char * codeset = "UTF-8";
 	KeySet * cks = ksDup (ks);
 
 	ksRewind (cks);
 
 	if (options & KDB_O_HEADER)
 	{
-		written += fprintf (stream, "<?xml version=\"1.0\" encoding=\"%s\"?>", codeset);
+		written += fprintf (stream, "<?xml version=\"1.0\" encoding=\"%s\"?>", "UTF-8");
 		if (~options & KDB_O_CONDENSED)
 			written += fprintf (stream, "\n<!-- Generated by Elektra API. Total of %d keys. -->\n", (int)cks->size);
 		if (~options & KDB_O_CONDENSED)
 			written += fprintf (stream,
-					    "<keyset xmlns=\"http://www.libelektra.org\"\n"
+					    "<keyset xmlns=\"https://www.libelektra.org\"\n"
 					    "\txmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
-					    "\txsi:schemaLocation=\"http://www.libelektra.org elektra.xsd\"\n");
+					    "\txsi:schemaLocation=\"https://www.libelektra.org elektra.xsd\"\n");
 		else
 			written += fprintf (stream,
-					    "<keyset xmlns=\"http://www.libelektra.org\""
+					    "<keyset xmlns=\"https://www.libelektra.org\""
 					    " xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\""
-					    " xsi:schemaLocation=\"http://www.libelektra.org elektra.xsd\"");
+					    " xsi:schemaLocation=\"https://www.libelektra.org elektra.xsd\"");
 	}
 	else
 		written += fprintf (stream, "<keyset");
@@ -452,18 +451,15 @@ int keyOutput (const Key * k, FILE * stream, option_t options)
 	time_t t;
 	size_t s;
 	char * tmc;
-	char * str;
 
 	size_t c;
-	char * com;
 
 	size_t n;
-	char * nam;
 
 	n = keyGetNameSize (k);
 	if (n > 1)
 	{
-		nam = (char *)elektraMalloc (n);
+		char * nam = (char *)elektraMalloc (n);
 		if (nam == NULL) return -1;
 		keyGetName (k, nam, n);
 
@@ -475,7 +471,7 @@ int keyOutput (const Key * k, FILE * stream, option_t options)
 	s = keyGetValueSize (k);
 	if (options & KEY_VALUE && s > 1)
 	{
-		str = (char *)elektraMalloc (s);
+		char * str = (char *)elektraMalloc (s);
 		if (str == NULL) return -1;
 		if (keyIsBinary (k))
 		{
@@ -501,7 +497,7 @@ int keyOutput (const Key * k, FILE * stream, option_t options)
 	c = keyGetCommentSize (k);
 	if (options & KEY_COMMENT && c > 1)
 	{
-		com = (char *)elektraMalloc (c);
+		char * com = (char *)elektraMalloc (c);
 		if (com == NULL) return -1;
 		keyGetComment (k, com, c);
 
