@@ -5,7 +5,7 @@ echo ELEKTRA CHECK FORMATTING
 echo
 
 command -v git >/dev/null 2>&1 || { echo "git command needed for this test, aborting" >&2; exit 0; }
-command -v clang-format-3.8 >/dev/null 2>&1 || { echo "clang-format-3.8 command needed for this test, aborting" >&2; exit 0; }
+command -v clang-format-3.8 >/dev/null 2>&1 || command -v clang-format >/dev/null 2>&1 || { echo "clang-format command needed for this test, aborting" >&2; exit 0; }
 
 cd "@CMAKE_SOURCE_DIR@"
 
