@@ -10,10 +10,12 @@ test -f "@PROJECT_BINARY_DIR@/compile_commands.json" || { echo "Compilation data
 
 cd "@CMAKE_SOURCE_DIR@" || exit
 oclint -p "@PROJECT_BINARY_DIR@" -enable-global-analysis -enable-clang-static-analyzer \
+	"@CMAKE_SOURCE_DIR@/src/libs/ease/array.c" \
 	"@CMAKE_SOURCE_DIR@/src/libs/ease/keyname.c" \
 	"@CMAKE_SOURCE_DIR@/src/libs/utility/text.c" \
-	"@CMAKE_SOURCE_DIR@/src/plugins/base64/"*.{c,cpp} \
+	"@CMAKE_SOURCE_DIR@/src/plugins/base64/"*.c \
 	"@CMAKE_SOURCE_DIR@/src/plugins/camel/camel.c" \
+	"@CMAKE_SOURCE_DIR@/src/plugins/directoryvalue/"*.c \
 	"@CMAKE_SOURCE_DIR@/src/plugins/mini/mini.c" \
 	"@CMAKE_SOURCE_DIR@/src/plugins/yamlcpp/"*.{c,cpp}
 exit_if_fail "OCLint found problematic code"
