@@ -8,12 +8,11 @@ resetGlobals()
 {
 	COMMAND=
 	RET=
-	ERRORS=
+	ERROR=
 	WARNINGS=
 	STDOUT=
 	STDOUTRE=
 	STDERR=
-	DIFF=
 	OUTBUF=
 	MOUNTPOINT=
 }
@@ -21,26 +20,28 @@ resetGlobals()
 writeBlock()
 {
 	OUTFILE="$1"
-	[ -n "$RET" ] && echo "RET: $RET" >> "$TMPFILE" || { [ -z "$ERRORS" ] && echo 'RET: 0' >> "$TMPFILE"; }
-	[ -n "$ERRORS" ] && echo "ERRORS: $ERRORS" >> "$TMPFILE"
-	[ -n "$WARNINGS" ] && echo "WARNINGS: $WARNINGS" >> "$TMPFILE"
-	[ -n "$DIFF" ] && echo "DIFF: $DIFF" >> "$TMPFILE"
-	[ -n "$STDERR" ] && echo "STDERR: $STDERR" >> "$TMPFILE"
-	if [ -n "$OUTBUF" ]; then echo "STDOUT: $OUTBUF" >> "$TMPFILE"
-	elif [ -n "$STDOUT" ]; then echo "STDOUT: $STDOUT" >> "$TMPFILE"
-	elif [ -n "$STDOUTRE" ]; then echo "STDOUT-REGEX: $STDOUTRE" >> "$TMPFILE"
+	[ -n "$RET" ] && printf 'RET: %s\n' $RET >> "$TMPFILE" || { [ -z "$ERROR" ] && printf 'RET: 0\n' >> "$TMPFILE"; }
+	[ -n "$ERROR" ] && printf 'ERROR: %s\n' "$ERROR" >> "$TMPFILE"
+	[ -n "$WARNINGS" ] && printf 'WARNINGS: %s\n' "$WARNINGS" >> "$TMPFILE"
+	[ -n "$STDERR" ] && printf 'STDERR: %s\n' "$STDERR" >> "$TMPFILE"
+	if [ -n "$OUTBUF" ]; then printf 'STDOUT: %s\n' "$OUTBUF" >> "$TMPFILE"
+	elif [ -n "$STDOUTRE" ]; then printf 'STDOUT-REGEX: %s\n' "$STDOUTRE" >> "$TMPFILE"
 	fi
 	COMMAND=$(sed s/sudo\ //g <<< "$COMMAND")
-	echo "< $COMMAND" >> "$TMPFILE"
+	while read -r cmd; do
+		printf '< %s\n' "$cmd" >> "$TMPFILE"
+	done <<< "$COMMAND"
 	resetGlobals
 }
 
 translate()
 {
 	TMPFILE=$(mktemp)
-	MOUNTPOINT=$(echo "$BUF" | head -n 1)
-	if grep -Eq 'Backup-and-Restore:' <<< "$MOUNTPOINT"; then echo "Mountpoint: $(cut -d ':' -f2 <<< "$MOUNTPOINT")" >> "$TMPFILE"
-	else echo 'Mountpoint: /examples' >> "$TMPFILE"
+	MOUNTPOINT=$(printf '%s' "$BUF" | head -n 1)
+	if grep -Eq 'Backup-and-Restore:' <<< "$MOUNTPOINT"; then
+		printf 'Mountpoint: %s\n' "$(cut -d ':' -f2 <<< "$MOUNTPOINT" | sed 's/^[[:space:]]*//')" >> "$TMPFILE"
+	else
+		printf 'Mountpoint: /examples\n' >> "$TMPFILE"
 	fi
 
 	resetGlobals
@@ -60,24 +61,18 @@ translate()
 				RET)
 					RET="$arg"
 					;;
-				STDOUT)
-				    	STDOUT="$arg"
-					;;
 				STDOUT-REGEX)
 					STDOUTRE="$arg"
 					;;
 				STDERR)
 					STDERR="$arg"
 					;;
-				ERRORS)
-					ERRORS="$arg"
+				ERROR)
+					ERROR="$arg"
 					;;
 				WARNINGS)
 					WARNINGS="$arg"
 					;;
-				DIFF)
-					DIFF="$arg"
-					;;
 				*)
 					;;
 			esac
@@ -91,8 +86,8 @@ translate()
 			while [ "${line: -1}" == '\' ];
 			do
 				read -r line
-				if [ "${line: -1}" == '\' ]; then COMMAND=$(printf '%s\\n%s' "$COMMAND" "${line%?}")
-				else COMMAND=$(printf '%s\\n%s\\n' "$COMMAND" "$line")
+				if [ "${line: -1}" == '\' ]; then COMMAND=$(printf '%s\n%s' "$COMMAND" "${line%?}")
+				else COMMAND=$(printf '%s\n%s\n' "$COMMAND" "$line")
 				fi
 			done
 			continue
@@ -113,7 +108,7 @@ SHELL_RECORDER_ERROR=0
 INBLOCK=0
 IFS=''
 
-MOUNTPOINTS_BACKUP=$("$KDBCOMMAND" mount)
+MOUNTPOINTS_BACKUP=$("$KDB" mount)
 
 while read -r line;
 do
@@ -125,17 +120,17 @@ done <<< "$BLOCKS"
 
 translate
 
-MOUNTPOINTS=$("$KDBCOMMAND" mount)
+MOUNTPOINTS=$("$KDB" mount)
 
 if [ "$MOUNTPOINTS_BACKUP" != "$MOUNTPOINT" ];
 then
 IFS='
 '
-	TOUMOUNT=$(diff <(echo "$MOUNTPOINTS_BACKUP") <(echo "$MOUNTPOINTS") | grep -Eo "^>.*")
+	TOUMOUNT=$(diff <(printf '%s' "$MOUNTPOINTS_BACKUP") <(printf '%s' "$MOUNTPOINTS") | grep -Eo "^>.*")
 	for line in $TOUMOUNT;
 	do
 		mp=$(sed -n 's/.*with name \(.*\)/\1/p' <<< "$line")
-		"$KDBCOMMAND" umount "$mp"
+		"$KDB" umount "$mp"
 	done
 fi
 
